#!/bin/python3
import os, sys

#classes used, some are used more like structures, but having classes allows for easier reapplication later
class coordinate:
    def __init__(self, x, y):
        self.x = x
        self.y = y
class item:
    def __init__(self, name, description, is_regen, is_edible):
        self.name = name
        self.description = description
        self.regen = is_regen
        self.edible = is_edible

class room:
    def __init__(self, name, loc, out_description, in_description, items, door_description, key):
        self.name = name
        self.loc = loc
        self.items = items
        self.in_description = in_description
        self.out_description = out_description
        self.door_description = door_description
        self.key = key

    def in_room(self, x):
        for i in self.items:
            if i.name == x:
                return True
        return False

class world:
    def __init__(self, name, sizex, sizey):
        self.name = name
        self.sizex = sizex
        self.sizey = sizey
        self.world_local = [[0 for i in range(sizex)] for i in range(sizey)]

class player:
    def __init__(self, location, hunger, inventory):
        self.location = location
        self.inventory = inventory
        self.hunger = hunger
    def decrease_hunger(self):
        self.hunger -= 1
    def increase_hunger(self,x):
        self.hunger += x
    def is_carrying(self, x):
        for i in self.inventory:
            if i.name == x:
                return True
        return False
    def print_items(self):
        if len(self.inventory) > 0:
            for i in self.inventory:
                print(i.description, end=', ')
            print("")
        else:
            print("nothing")
    def item_named(self, x):
        if self.is_carrying(x):
            for i in self.inventory:
                if i.name == x:
                    return i


#GENERIC FUNCTIONS
def door_check(room, player):
    if room.key == '':
        return 0
    elif room.key != '':
        if player.is_carrying(room.key):
            print('You use your %s to open the door' %room.key)
            return 0
        print(room.door_description)
        return 1
    else:
        print(room.door_description)
        return 1
    

def north(world, player):
    r = world.world_local[player.location.x][player.location.y]
    nr = world.world_local[int(r.loc.x)][int(r.loc.y) + 1]
    if nr == 0:
        print("You cannot go that way")
    elif door_check(nr, player) == 0:
        player.location.x = r.loc.x
        player.location.y = r.loc.y+1
        player.decrease_hunger()
        return 1

def south(world, player):
    r = world.world_local[player.location.x][player.location.y]
    nr = world.world_local[int(r.loc.x)][int(r.loc.y) - 1]
    if nr == 0:
        print("You cannot go that way")
    elif door_check(nr, player) == 0:
        player.location.x = r.loc.x
        player.location.y = r.loc.y-1
        player.decrease_hunger()
        return 1

def east(world, player):
    r = world.world_local[player.location.x][player.location.y]
    nr = world.world_local[int(r.loc.x) + 1][int(r.loc.y)]
    if nr == 0:
        print("You cannot go that way")
    elif door_check(nr, player) == 0:
        player.location.x = r.loc.x + 1
        player.location.y = r.loc.y
        player.decrease_hunger()
        return 1

def west(world, player):
    r = world.world_local[player.location.x][player.location.y]
    nr = world.world_local[int(r.loc.x) - 1][int(r.loc.y)]
    if nr == 0:
        print("You cannot go that way")
    elif door_check(nr, player) == 0:
        player.location.x = r.loc.x - 1
        player.location.y = r.loc.y
        player.decrease_hunger()
        return 1

def pickup(world, player, x):
    r = world.world_local[player.location.x][player.location.y]
    t = 0
    for i in r.items:
        if x == i.name:
            j = r.items.index(i)
            player.inventory.append(r.items[j])
            print("Picked up %s" %x)
            if r.items[j].regen == 0:
                r.items.pop(j)
            del j
            t = 1
    if t != 1:
        print("That item doesn't exist here")

def inventory(world, player):
    print("You currently are carrying: ", end='')
    player.print_items()

def drop(world, player, x):
    r = world.world_local[player.location.x][player.location.y]
    if player.is_carrying(x):
        r.items.append(player.item_named(x))
        player.inventory.pop(player.inventory.index(player.item_named(x)))
        print("Dropped %s" %x)
    else:
        print("You are carrying no such item")
        

def look(world, player):
    return 1

def eat(world, player, x):
    r = world.world_local[player.location.x][player.location.y]
    for i in player.inventory:
        if x == i.name:
            if i.edible == 1:
                player.increase_hunger(5)
                print("Yum")
                player.inventory.pop(player.inventory.index(i))
            elif x == 'eggs':
                print("These eggs don't seem edible")
            else:
                print("You cannot eat %s" %i.name)
            return 2
    for j in r.items:
        if x == j.name:
            print("You must pickup %s before you can try to eat it" %j.name)
            return 2  
    print("I cannot seem to find this item")

def hunger(world, player):
    print(player.hunger)

def use(world, player, x):
    r = world.world_local[player.location.x][player.location.y]
    if r.in_room(x):
        if x in special_funcs:
            special_funcs[x](world,player)
        else:
            print("You cannot use that item")
            return 2
    else:
        print("There is no %s to use here" %x)
        return 2

def quit(world, player):
    print("Goodbye!")
    sys.exit()

def help(world, player):
    print("This is a text based adventure! Here are some hints on how to get around: \n You can type which direction you wish to travel (north, south, east, or west). \n You can pickup (or take) items. \n You can eat an item. \n There is one item that you can use \n Typing 'hunger' will tell you the numbers of turns left before you starve. \n Typing 'inventory' will tell you what your are carrying \n If you are feeling lost, look around. \n When you are feeling done, you can always quit")

#GAME SPECIFIC FUNCTIONS
def computer(world, player):
    if player.is_carrying('floppy disk'):    
        if os.name == 'nt':
            print("Have fun! (To exit the game use ^C)") 
            os.system('py xyppy.py zork1.z5')
            look(world, player)
        else:
            print("Have fun! (To exit the game use ^C)") 
            os.system('python xyppy.py zork1.z5')
            look(world, player)
    else:
        print("You have nothing to use this computer with")

#FUNCTION DICTIONARY
funcs = {'north':north, 'n': north, 'south':south, 's':south, 'east':east, 'e':east, 'west':west, 'w':west, 'pickup':pickup, 'take':pickup, 'inventory':inventory, 'look':look, 'l':look, 'eat':eat, 'hunger':hunger, 'use':use, 'drop':drop, 'take':pickup, 'help':help, 'quit':quit}
special_funcs = {'computer':computer}

items = []
rooms = []
win = []

def file_len(fname):
    f = open(fname)
    for i, l in enumerate(f):
            pass
    return i + 1

def init_world(fname):
    global items
    global rooms
    global world
    global player
    global win

    f = open(fname,"r")
    line = f.readline()
    l = 0
    room_num = 0
    current_line = 0
    while line:
        if line == "---\n": #Delimiter between sections in config file is ---
            l += 1
            line = f.readline()
            continue
        temp = str(line).split("/")
        if l == 0: #Firstly, Items are Defined
            items.append(item(temp[1],temp[2],int(temp[3]), int(temp[4])))
        if l == 1: #Second, Rooms are Defined
            temp1 = temp[2].split(",")
            rooms.append(room(temp[1],(coordinate(int(temp1[0]),int(temp1[1]))),temp[3],temp[4],[],temp[6],temp[7]))
            temp2 = temp[5].split(",")
            for j in temp2:
                for i in items:
                    if j == i.name:
                        rooms[room_num].items.append(i)
            room_num += 1
        if l == 2: #Third, the World is defined
            temp1 = temp[2].split(",")
            world = world(temp[1],int(temp1[0]),int(temp1[1]))
        if l == 3: #Fourth, the Player is defined
            temp1 = temp[1].split(",")
            player = player(coordinate(int(temp1[0]),int(temp1[1])), int(temp[2]),[])
        if l == 4: #Fifth, the winning itemset is defined
            temp1 = temp[1].split(",")
            for j in temp1:
                win.append(j)
        line = f.readline()
        current_line += 1
    for i in rooms:
        world.world_local[i.loc.x][i.loc.y] = i    
    f.close()

def list_rooms():
    r = world.world_local[player.location.x][player.location.y]
    if world.world_local[int(r.loc.x)][int(r.loc.y) + 1] != 0:
        print("To the north you see %s" %world.world_local[player.location.x][player.location.y + 1].out_description)
    if world.world_local[int(r.loc.x)][int(r.loc.y) - 1] != 0:
        print("To the south you see %s" %world.world_local[player.location.x][player.location.y - 1].out_description)
    if world.world_local[int(r.loc.x) + 1][int(r.loc.y)] != 0:
        print("To the east you see %s" %world.world_local[player.location.x + 1][player.location.y].out_description)
    if world.world_local[int(r.loc.x) - 1][int(r.loc.y)] != 0:
        print("To the west you see %s" %world.world_local[player.location.x - 1][player.location.y].out_description)

def print_items(room):
    for k,i in zip(range(len(room.items)),room.items):
        if k < len(room.items) - 1:
            print(i.description, end = ', ')
        else:
            print(i.description, end = ' ')
            
    print('')
            
def check_win():
    global win
    global player

    i = 1

    for j in win:
        if not player.is_carrying(j):
            break
        else:
            i += 1
    if i >= len(win):
        return 1
def run_game():
    print("You wake dazed in field. What happened last night? You remember going all around campus. Your pack is empty, and you realize that you must find all of your items soon or you might loose your job running the field after every point at the football games. You also haven't eaten in a while. Perhaps you should find some food. \n")
    while 1:
        run = 1
        inp = ""
        r = world.world_local[player.location.x][player.location.y]
        print("THE  %s \n" %r.name.upper())
        print("%s" %r.in_description)
        print("Here you find: ", end = '')
        print_items(r)
        list_rooms()
        if player.hunger == 0:
            print("A sense of doom takes over. You realize you haven't eaten anything in a long time. Your mind begins to turn to madness as you drop to the ground")
            break
        elif player.hunger == 1:
            print("You grimace with a twang of hunger")
        elif player.hunger == 6:
            print("Your stomach grumbles. Perhaps you should find something to eat.")
        while 1:
            inp = input(">_ ").lower().strip()
            inp = inp.split(' ',1)
            if inp[0] in funcs:
                if len(inp) == 1:
                    try:
                        long_out = funcs[inp[0]](world, player)
                        if long_out == 1:
                            break
                    except:
                        print('Input Not Understood')
                elif len(inp) > 1:
                    try:
                        long_out = funcs[inp[0]](world, player, inp[1])
                        if long_out == 1:
                            break
                    except:
                        print('Input Not Understood')
            else:
                print('Input Not Understood')
            if check_win() == 1:
                break
        if check_win() == 1:
            print("CONGRATS! You have found all the items. You Win!")
            break

def main ():
    init_world("./config") #Initialized Game from Config File.
    run_game()

main()
